#include <stdio.h>
#include <time.h>

#define OK     1
#define FAIL   0
#define ERROR -1

#define CORRECT   1
#define INCORRECT 0

#define QUESTION_NUM  1
#define QUESTION_COL  0

#define Q_LEN 50


char colours[][9] = { "black",
                      "brown",
                      "red",
                      "orange",
                      "yellow",
                      "green",
                      "blue",
                      "violet",
                      "grey",
                      "white"
                    };
                    
char altcolours[][9] = { "",
                        "",
                        "",
                        "",
                        "",
                        "",
                        "",
                        "purple",
                        "gray",
                        ""
                      };
                      
/* int stdvals[] = {}; */

                      
                      
                      
typedef struct session_s {
   int questions;
   int correctCount;
   int incorrectCount;
   int answer;
   int questionType;
   char student[33];
} session_t;              


                             
                      
/****************************** QUESTIONS *************************************/

/*
   Question the user about a colour
*/                      
int colourQuestion( char * quote, int buffSize, int number) {

   if ( (  number >= 0) && ( number <= 9 ) ) {
      snprintf(quote, buffSize, "What is the value of the color %s?", colours[number]);
      return OK;
   } else {
      return FAIL;
   }
      
}                                          


/*
  Question the user about a value
*/
int numberQuestion( char * quote, int buffSize, int number) {

   if ( ( number >= 0) && ( number <= 9 ) ) {
      snprintf(quote, buffSize, "What is the colour for the value %d?", number);
      return OK;
   } else {
      return FAIL;
   }   
}                                          

/*
  Ask the user a question, return value is the band number in question.
*/
int GenQuestion( char * quote, int buffSize, int questionType) {

    int value;
    
    value = (rand() % 10);

    if ( questionType == QUESTION_COL ) {
       if (colourQuestion( quote, buffSize, value ) != OK)
         return ERROR;
    } else {
       if (numberQuestion( quote, buffSize, value ) != OK)
         return ERROR;
    }
    
    return value;
}

/*********************************** ANSWERS ***********************************/

/*
  check an answer, the input must be a stripped string.
  e.g.  "1"  or "red"
*/
int checkAnswer (int questionType, int value, char * reply ) {

   if (questionType == QUESTION_NUM) {
     return checkForColour(value, reply);
   } else {
     return checkForNumber(value, reply);
   }
   
}

/*
  check for a numerical answer
*/
int checkForNumber(int value, char * reply) {

   if (atoi(reply) == value) 
      return CORRECT;
   else
      return INCORRECT;
      
}

/*
  checks for a string answer
*/
int checkForColour(int value, char * reply) {

   if (strcmp(colours[value], reply) == 0)
     return CORRECT;
   else
     if (strcmp(altcolours[value], reply) == 0)
       return CORRECT;
     else
       return INCORRECT;

}

/*
  calculate the score.
*/
int calcScore( int correct, int incorrect ) {

  return (int)( ( (float)correct / ( (float)correct + (float)incorrect ) ) * (float)100 );

}

/********************************* struct stuff ****************************/

/*
  set up a question session
*/
void initSession( session_t * session ) {

  time_t currtime;
     
  time(&currtime);
  srand(currtime);
  session->correctCount   = 0;
  session->incorrectCount = 0;
  return;
  
}

/*
  start a session with n questions
*/
void startSession( session_t * session, int questions ) {

  initSession(session);
  session->questions = questions;
  return;

}


/*
  generate a question string
*/
void sessionQuestion( session_t * session, char * question ) {

  session->questionType = rand() % 2;
  session->answer       = GenQuestion( question, Q_LEN, session->questionType);
  session->questions--;
  return;

}

/*
  check an answer, generate a reply.
*/
void sessionAnswer( session_t * session, char * user, char * computer) {

   if( checkAnswer (session->questionType, session->answer, user ) == CORRECT) {
     snprintf(computer, Q_LEN, "Correct.  ");
     session->correctCount++;
   } else {
     if (session->questionType == QUESTION_COL ) {
        snprintf(computer, Q_LEN, "No, the answer was %d.", session->answer);
     } else {
        snprintf(computer, Q_LEN, "No, the answer was %s.", colours[session->answer]);     
     }
     session->incorrectCount++;
   }
   return;

}

/*
  generate a score string.
*/
void sessionScore( session_t * session, char * computer) {

   snprintf(computer, Q_LEN, "Your score was: %d%%", calcScore(session->correctCount, session->incorrectCount));
   return;
   
}

/*
  is the session still active?
*/
int sessionActive( session_t * session ) {

   return (session->questions >= 0 );
   
}

/********************************************************************************/

/*

doresistors n



int main(void) {

 session_t session;

 char computer[Q_LEN];
 char user[Q_LEN];
 
 int temp;
 
 //-------------------           

 printf("How many questions would you like to do?");
 scanf("%d", &temp);
 printf("\n");
 
 startSession( &session, temp );
 
 if (!sessionActive( &session ))
    return 0;
 
 sessionQuestion( &session, computer );   
 printf("%s\n", computer);
   
 do {
 
   scanf("%s",  &user);   // !!!???!!! accident waiting to happen   
   sessionAnswer( &session,  user, computer);
   printf("%s\n", computer);
   
   sessionQuestion( &session, computer ); 
   
   if (sessionActive( &session )) {       
     printf("%s\n", computer);
   }
   
 }  while (sessionActive( &session ));
 
 sessionScore(&session, computer);
 printf("%s\n", computer);
 
 return 0;
}

*/
